package jalview.ext.archaeopteryx;

import jalview.ext.treeviewer.TreeFrameI;
import jalview.ext.treeviewer.TreeI;
import jalview.ext.treeviewer.TreeNodeI;

import java.util.Iterator;

import org.forester.archaeopteryx.Archaeopteryx;
import org.forester.phylogeny.Phylogeny;
import org.forester.phylogeny.PhylogenyMethods;
import org.forester.phylogeny.PhylogenyNode;

public class Tree implements TreeI
{
  private final Phylogeny tree;

  // alternative to static maps in TreeNode
  // private Map<PhylogenyNode, TreeNodeI> originalNodes = new HashMap<>(500);
  // private Map<TreeNodeI, PhylogenyNode> wrappedNodes = new HashMap<>(500);

  public Tree()
  {
    tree = new Phylogeny();
  }
  public Tree(Phylogeny aptxTree)
  {
    tree = aptxTree;
    wrapAllTreeNodes();

  }

  private void wrapAllTreeNodes()
  {
    for (Iterator<PhylogenyNode> iterator = tree
            .iteratorPostorder(); iterator.hasNext();)
    {
      PhylogenyNode foresterNode = iterator.next();
      TreeNodeI treeNode = TreeNode
              .getUniqueWrapper(foresterNode);

    }
  }

  @Override
  public TreeNodeI getRoot()
  {
    TreeNodeI root = TreeNode.getUniqueWrapper(tree.getRoot());
    return root;
  }

  @Override
  public void setTreeName(String name)
  {
    tree.setName(name);

  }

  @Override
  public TreeNodeI getNodeWithName(String name)
  {
    return TreeNode.getUniqueWrapper(tree.getNode(name));

  }

  @Override
  public String[] getAllLeafNames()
  {
    return tree.getAllExternalNodeNames();
  }

  @Override
  public void setRerootable(boolean b)
  {
    tree.setRerootable(b);

  }

  @Override
  public void setRooted(boolean b)
  {
    tree.setRooted(b);

  }

  @Override
  public boolean isEmpty()
  {
    return tree.isEmpty();
  }

  @Override
  public String getTreeName()
  {
    return tree.getName();
  }

  @Override
  public void setRoot(TreeNodeI rootNode)
  {
    PhylogenyNode treeRoot = TreeNode.unwrapNode(rootNode);
    tree.setRoot(treeRoot);
    wrapAllTreeNodes();

  }

  @Override
  public double getHeight(boolean adjustForCollapsedSubtrees)
  {
    return tree.calculateHeight(adjustForCollapsedSubtrees);
  }

  @Override
  public Iterator<TreeNodeI> iterateInPreOrder()
  {
    Iterator<TreeNodeI> iter = new TreeIterator(
            tree.iteratorPreorder());
    return iter;
  }

  @Override
  public Iterator<TreeNodeI> iterateInLevelOrder()
  {
    Iterator<TreeNodeI> iter = new TreeIterator(
            tree.iteratorLevelOrder());
    return iter;
  }

  @Override
  public Iterator<TreeNodeI> iterateInPostOrder()
  {
    Iterator<TreeNodeI> iter = new TreeIterator(
            tree.iteratorPostorder());
    return iter;
  }

  @Override
  public TreeNodeI getFurthestNode()
  {
    PhylogenyNode furthestNode = PhylogenyMethods
            .calculateNodeWithMaxDistanceToRoot(tree);
    return TreeNode.getUniqueWrapper(furthestNode);
  }

  @Override
  public TreeFrameI createTreeViewerFromTree(String instanceTitle)
  {
    return new AptxFrame(Archaeopteryx.createApplication(tree,
            AptxInit.APTX_CONFIG,
            instanceTitle));
  }


}
